# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import testtools

from glareclient.common import utils


class TestUtils(testtools.TestCase):

    def test_make_size_human_readable(self):
        self.assertEqual("106B", utils.make_size_human_readable(106))
        self.assertEqual("1000kB", utils.make_size_human_readable(1024000))
        self.assertEqual("1MB", utils.make_size_human_readable(1048576))
        self.assertEqual("1.4GB", utils.make_size_human_readable(1476395008))
        self.assertEqual("9.3MB", utils.make_size_human_readable(9761280))
        self.assertEqual("0B", utils.make_size_human_readable(None))



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from glareclient import exc
from glareclient.osc.v1 import blobs as osc_blob
from glareclient.tests.unit.osc.v1 import fakes
from glareclient.v1 import artifacts as api_art
import testtools


class TestUpload(testtools.TestCase):

    def setUp(self):
        self.mock_app = mock.Mock()
        self.mock_args = mock.Mock()
        self.mock_manager = mock.Mock()
        self.mock_manager.artifacts.get.return_value = {'image': {}}
        super(TestUpload, self).setUp()

    @mock.patch('glareclient.osc.v1.blobs.progressbar')
    @mock.patch('glareclient.osc.v1.blobs.sys')
    @mock.patch('glareclient.osc.v1.blobs.open', create=True)
    @mock.patch('glareclient.common.utils.get_artifact_id')
    def test_upload_file_progress(self, mock_get_id,
                                  mock_open, mock_sys, mock_progressbar):
        mock_parsed_args = mock.Mock(name='test-id',
                                     id=True,
                                     blob_property='image',
                                     file='/path/file',
                                     progress=True,
                                     content_type='application/test',
                                     type_name='test-type')
        mock_get_id.return_value = 'test-id'
        cli = osc_blob.UploadBlob(self.mock_app, self.mock_args)
        cli.app.client_manager.artifact = self.mock_manager
        cli.dict2columns = mock.Mock(return_value=42)
        self.assertEqual(42, cli.take_action(mock_parsed_args))
        cli.dict2columns.assert_called_once_with({'blob_property': 'image'})
        upload_args = ['test-id', 'image',
                       mock_progressbar.VerboseFileWrapper.return_value]
        upload_kwargs = {'content_type': 'application/test',
                         'type_name': 'test-type'}
        self.mock_manager.artifacts.upload_blob.\
            assert_called_once_with(*upload_args, **upload_kwargs)

    @mock.patch('glareclient.osc.v1.blobs.sys')
    @mock.patch('glareclient.osc.v1.blobs.open', create=True)
    @mock.patch('glareclient.common.utils.get_artifact_id')
    def test_upload_file_no_progress(self, mock_get_id, mock_open, mock_sys):
        mock_parsed_args = mock.Mock(name='test-id',
                                     id=True,
                                     blob_property='image',
                                     progress=False,
                                     file='/path/file',
                                     content_type='application/test',
                                     type_name='test-type')
        mock_get_id.return_value = 'test-id'
        cli = osc_blob.UploadBlob(self.mock_app, self.mock_args)
        cli.app.client_manager.artifact = self.mock_manager
        cli.dict2columns = mock.Mock(return_value=42)
        self.assertEqual(42, cli.take_action(mock_parsed_args))
        cli.dict2columns.assert_called_once_with({'blob_property': 'image'})
        upload_args = ['test-id', 'image', mock_open.return_value]
        upload_kwargs = {'content_type': 'application/test',
                         'type_name': 'test-type'}
        self.mock_manager.artifacts.upload_blob.\
            assert_called_once_with(*upload_args, **upload_kwargs)

    @mock.patch('glareclient.osc.v1.blobs.sys')
    @mock.patch('glareclient.common.utils.get_artifact_id')
    def test_upload_file_stdin(self, mock_get_id, mock_sys):
        mock_sys.stdin.isatty.return_value = False
        mock_parsed_args = mock.Mock(name='test-id',
                                     id=True,
                                     blob_property='image',
                                     progress=False,
                                     file=None,
                                     content_type='application/test',
                                     type_name='test-type')
        mock_get_id.return_value = 'test-id'
        cli = osc_blob.UploadBlob(self.mock_app, self.mock_args)
        cli.app.client_manager.artifact = self.mock_manager
        cli.dict2columns = mock.Mock(return_value=42)
        self.assertEqual(42, cli.take_action(mock_parsed_args))
        cli.dict2columns.assert_called_once_with({'blob_property': 'image'})
        upload_args = ['test-id', 'image', mock_sys.stdin]
        upload_kwargs = {'content_type': 'application/test',
                         'type_name': 'test-type'}
        self.mock_manager.artifacts.upload_blob.\
            assert_called_once_with(*upload_args, **upload_kwargs)

    @mock.patch('glareclient.osc.v1.blobs.sys')
    def test_upload_file_stdin_isatty(self, mock_sys):
        mock_sys.stdin.isatty.return_value = True
        mock_parsed_args = mock.Mock(id='test-id',
                                     blob_property='image',
                                     progress=False,
                                     file=None,
                                     content_type='application/test',
                                     type_name='test-type')
        cli = osc_blob.UploadBlob(self.mock_app, self.mock_args)
        cli.app.client_manager.artifact = self.mock_manager
        self.assertRaises(exc.CommandError, cli.take_action, mock_parsed_args)


class TestBlobs(fakes.TestArtifacts):
    def setUp(self):
        super(TestBlobs, self).setUp()
        self.blob_mock = \
            self.app.client_manager.artifact.blobs
        self.http = mock.MagicMock()


class TestDownloadBlob(TestBlobs):
    def setUp(self):
        super(TestDownloadBlob, self).setUp()
        self.blob_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_blob.DownloadBlob(self.app, None)

        self.COLUMNS = ('blob_property', 'id', 'name',
                        'size', 'status', 'version')

    def test_download_exception(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--blob-property', 'blob',
                   '--file', None]
        verify = [('type_name', 'images')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        with testtools.ExpectedException(SystemExit):
            self.cmd.take_action(parsed_args)

    def test_download_blob(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--blob-property', 'blob',
                   '--file', '/path/to/file']
        verify = [('type_name', 'images')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        self.cmd.take_action(parsed_args)

    def test_download_without_blob_property(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--file', '/path/to/file']
        verify = [('type_name', 'images')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        self.cmd.take_action(parsed_args)

    def test_download_progress(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--file', '/path/to/file',
                   '--progress']
        verify = [('type_name', 'images')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        self.cmd.take_action(parsed_args)


class TestAddLocation(TestBlobs):
    def setUp(self):
        super(TestAddLocation, self).setUp()
        self.blob_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_blob.AddLocation(self.app, None)
        self.COLUMNS = ('blob_property', 'content_type', 'external',
                        'md5', 'sha1', 'sha256', 'size', 'status', 'url')

    def test_add_location(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id',
                   '--url', 'fake_url',
                   '--md5', "35d83e8eedfbdb87ff97d1f2761f8ebf",
                   '--sha1', "942854360eeec1335537702399c5aed940401602",
                   '--sha256', "d8a7834fc6652f316322d80196f6dcf2"
                               "94417030e37c15412e4deb7a67a367dd"]
        verify = [('type_name', 'images'), ('url', 'fake_url')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.COLUMNS, columns)

    def test_add_dict_location(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id',
                   '--blob-property', 'nested_templates/blob',
                   '--url', 'fake_url',
                   '--md5', "35d83e8eedfbdb87ff97d1f2761f8ebf",
                   '--sha1', "942854360eeec1335537702399c5aed940401602",
                   '--sha256', "d8a7834fc6652f316322d80196f6dcf2"
                               "94417030e37c15412e4deb7a67a367dd"]
        verify = [('type_name', 'images'), ('url', 'fake_url')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        self.app.client_manager.artifact.artifacts.get = \
            lambda *args, **kwargs: {
                'nested_templates': {'blob': fakes.blob_fixture}
            }
        columns, data = self.cmd.take_action(parsed_args)
        self.app.client_manager.artifact.artifacts.get = fakes.mock_get
        self.assertEqual(self.COLUMNS, columns)


class TestRemoveLocation(TestBlobs):
    def setUp(self):
        super(TestRemoveLocation, self).setUp()
        self.blob_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_blob.RemoveLocation(self.app, None)

    def test_remove_location(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        self.assertIsNone(self.cmd.take_action(parsed_args))

    def test_remove_dict_location(self):
        arglist = ['images', '--blob-property', 'nested_templates/blob',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        self.assertIsNone(self.cmd.take_action(parsed_args))



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

FIXTURE_SCHEMA = {
    u'name': u'sample_artifact',
    u'properties': {
        u'activated_at': {
            u'description': u'Datetime when artifact has became active.',
            u'filter_ops': [u'eq',
                            u'neq',
                            u'in',
                            u'gt',
                            u'gte',
                            u'lt',
                            u'lte'],
            u'format': u'date-time',
            u'glareType': u'DateTime',
            u'readOnly': True,
            u'required_on_activate': False,
            u'sortable': True,
            u'type': [u'string',
                      u'null']},
        u'created_at': {
            u'description': u'Datetime when artifact has been created.',
            u'filter_ops': [u'eq',
                            u'neq',
                            u'in',
                            u'gt',
                            u'gte',
                            u'lt',
                            u'lte'],
            u'format': u'date-time',
            u'glareType': u'DateTime',
            u'readOnly': True,
            u'sortable': True,
            u'type': u'string'},
        u'description': {u'default': u'',
                         u'description': u'Artifact description.',
                         u'filter_ops': [u'eq',
                                         u'neq',
                                         u'in'],
                         u'glareType': u'String',
                         u'maxLength': 4096,
                         u'mutable': True,
                         u'required_on_activate': False,
                         u'type': [u'string',
                                   u'null']},
        u'icon': {u'additionalProperties': False,
                  u'description': u'Artifact icon.',
                  u'filter_ops': [],
                  u'glareType': u'Blob',
                  u'properties': {u'md5': {u'type': [u'string', u'null']},
                                  u'sha1': {u'type': [u'string', u'null']},
                                  u'sha256': {u'type': [u'string', u'null']},
                                  u'content_type': {u'type': u'string'},
                                  u'external': {u'type': u'boolean'},
                                  u'size': {u'type': [u'number',
                                                      u'null']},
                                  u'status': {u'enum': [u'saving',
                                                        u'active',
                                                        u'pending_delete'],
                                              u'type': u'string'}},
                  u'required': [u'size',
                                u'md5', u'sha1', u'sha256',
                                u'external',
                                u'status',
                                u'content_type'],
                  u'required_on_activate': False,
                  u'type': [u'object',
                            u'null']},
        u'id': {u'description': u'Artifact UUID.',
                u'filter_ops': [u'eq',
                                u'neq',
                                u'in'],
                u'glareType': u'String',
                u'maxLength': 255,
                u'pattern': u'^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-'
                            u'fA-F]){4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$',
                u'readOnly': True,
                u'sortable': True,
                u'type': u'string'},
        u'license': {u'description': u'Artifact license type.',
                     u'filter_ops': [u'eq',
                                     u'neq',
                                     u'in'],
                     u'glareType': u'String',
                     u'maxLength': 255,
                     u'required_on_activate': False,
                     u'type': [u'string',
                               u'null']},
        u'license_url': {u'description': u'URL to artifact license.',
                         u'filter_ops': [u'eq',
                                         u'neq',
                                         u'in'],
                         u'glareType': u'String',
                         u'maxLength': 255,
                         u'required_on_activate': False,
                         u'type': [u'string',
                                   u'null']},
        u'metadata': {u'additionalProperties': {u'type': u'string'},
                      u'default': {},
                      u'description': u'Key-value dict with useful information'
                                      u'about an artifact.',
                      u'filter_ops': [u'eq',
                                      u'neq'],
                      u'glareType': u'StringDict',
                      u'maxProperties': 255,
                      u'required_on_activate': False,
                      u'type': [u'object',
                                u'null']},
        u'name': {u'description': u'Artifact Name.',
                  u'filter_ops': [u'eq',
                                  u'neq',
                                  u'in'],
                  u'glareType': u'String',
                  u'maxLength': 255,
                  u'required_on_activate': False,
                  u'sortable': True,
                  u'type': u'string'},
        u'owner': {u'description': u'ID of user/tenant who uploaded artifact.',
                   u'filter_ops': [u'eq',
                                   u'neq',
                                   u'in'],
                   u'glareType': u'String',
                   u'maxLength': 255,
                   u'readOnly': True,
                   u'required_on_activate': False,
                   u'sortable': True,
                   u'type': u'string'},
        u'provided_by': {u'additionalProperties': False,
                         u'description': u'Info about artifact authors.',
                         u'filter_ops': [u'eq',
                                         u'neq',
                                         u'in'],
                         u'glareType': u'StringDict',
                         u'maxProperties': 255,
                         u'properties': {u'company': {u'type': u'string'},
                                         u'href': {u'type': u'string'},
                                         u'name': {u'type': u'string'}},
                         u'required_on_activate': False,
                         u'type': [u'object',
                                   u'null']},
        u'release': {u'default': [],
                     u'description': u'Target OpenStack release for artifact. '
                                     u'It is usually the same when artifact '
                                     u'was uploaded.',
                     u'filter_ops': [u'eq',
                                     u'neq',
                                     u'in'],
                     u'glareType': u'StringList',
                     u'items': {u'type': u'string'},
                     u'maxItems': 255,
                     u'required_on_activate': False,
                     u'type': [u'array',
                               u'null'],
                     u'unique': True},
        u'status': {u'default': u'drafted',
                    u'description': u'Artifact status.',
                    u'enum': [u'drafted',
                              u'active',
                              u'deactivated',
                              u'deleted'],
                    u'filter_ops': [u'eq',
                                    u'neq',
                                    u'in'],
                    u'glareType': u'String',
                    u'sortable': True,
                    u'type': u'string'},
        u'supported_by': {u'additionalProperties': {u'type': u'string'},
                          u'description': u'Info about persons who '
                                          u'responsible for artifact support',
                          u'filter_ops': [u'eq',
                                          u'neq',
                                          u'in'],
                          u'glareType': u'StringDict',
                          u'maxProperties': 255,
                          u'required': [u'name'],
                          u'required_on_activate': False,
                          u'type': [u'object',
                                    u'null']},
        u'tags': {u'default': [],
                  u'description': u'List of tags added to Artifact.',
                  u'filter_ops': [u'eq',
                                  u'neq',
                                  u'in'],
                  u'glareType': u'StringList',
                  u'items': {u'type': u'string'},
                  u'maxItems': 255,
                  u'mutable': True,
                  u'required_on_activate': False,
                  u'type': [u'array',
                            u'null']},
        u'updated_at': {
            u'description': u'Datetime when artifact has been updated '
                            u'last time.',
            u'filter_ops': [u'eq',
                            u'neq',
                            u'in',
                            u'gt',
                            u'gte',
                            u'lt',
                            u'lte'],
            u'format': u'date-time',
            u'glareType': u'DateTime',
            u'readOnly': True,
            u'sortable': True,
            u'type': u'string'},
        u'version': {u'default': u'0.0.0',
                     u'description': u'Artifact version(semver).',
                     u'filter_ops': [u'eq',
                                     u'neq',
                                     u'in',
                                     u'gt',
                                     u'gte',
                                     u'lt',
                                     u'lte'],
                     u'glareType': u'String',
                     u'pattern': u'/^([0-9]+)\\.([0-9]+)\\.([0-9]+)(?:-'
                                 u'([0-9A-Za-z-]+(?:\\.[0-9A-Za-z-]+)*))?'
                                 u'(?:\\+[0-9A-Za-z-]+)?$/',
                     u'required_on_activate': False,
                     u'sortable': True,
                     u'type': u'string'},
        u'visibility': {u'default': u'private',
                        u'description': u'Artifact visibility that defines if '
                                        u'artifact can be available to other '
                                        u'users.',
                        u'filter_ops': [u'eq'],
                        u'glareType': u'String',
                        u'maxLength': 255,
                        u'sortable': True,
                        u'type': u'string'},
        u'image': {u'additionalProperties': False,
                   u'description': u'Image binary.',
                   u'filter_ops': [],
                   u'glareType': u'Blob',
                   u'properties': {
                       u'md5': {u'type': [u'string', u'null']},
                       u'sha1': {u'type': [u'string', u'null']},
                       u'sha256': {u'type': [u'string', u'null']},
                       u'content_type': {u'type': u'string'},
                       u'external': {u'type': u'boolean'},
                       u'size': {u'type': [u'number',
                                           u'null']},
                       u'status': {u'enum': [u'saving',
                                             u'active',
                                             u'pending_delete'],
                                   u'type': u'string'}},
                   u'required': [u'size',
                                 u'md5', u'sha1', u'sha256',
                                 u'external',
                                 u'status',
                                 u'content_type'],
                   u'required_on_activate': False,
                   u'type': [u'object', u'null']},
        u'package': {
            u'additionalProperties': False,
            u'description': u'Murano Package binary.',
            u'filter_ops': [],
            u'glareType': u'Blob',
            u'properties': {u'md5': {u'type': [u'string', u'null']},
                            u'sha1': {u'type': [u'string', u'null']},
                            u'sha256': {u'type': [u'string', u'null']},
                            u'content_type': {u'type': u'string'},
                            u'external': {u'type': u'boolean'},
                            u'size': {u'type': [u'number',
                                                u'null']},
                            u'status': {u'enum': [u'saving',
                                                  u'active',
                                                  u'pending_delete'],
                                        u'type': u'string'}},
            u'required': [u'size',
                          u'md5', u'sha1', u'sha256',
                          u'external',
                          u'status',
                          u'content_type'],
            u'required_on_activate': False,
            u'type': [u'object',
                      u'null']},
        u'environment': {
            u'additionalProperties': False,
            u'description': u'Heat Environment text body.',
            u'filter_ops': [],
            u'glareType': u'Blob',
            u'properties': {u'md5': {u'type': [u'string', u'null']},
                            u'sha1': {u'type': [u'string', u'null']},
                            u'sha256': {u'type': [u'string', u'null']},
                            u'content_type': {u'type': u'string'},
                            u'external': {u'type': u'boolean'},
                            u'size': {u'type': [u'number',
                                                u'null']},
                            u'status': {u'enum': [u'saving',
                                                  u'active',
                                                  u'pending_delete'],
                                        u'type': u'string'}},
            u'required': [u'size',
                          u'md5', u'sha1', u'sha256',
                          u'external',
                          u'status',
                          u'content_type'],
            u'type': [u'object',
                      u'null']},
        u'blob': {
            u'additionalProperties': False,
            u'description': u'I am Blob',
            u'filter_ops': [],
            u'glareType': u'Blob',
            u'mutable': True,
            u'properties': {
                u'md5': {u'type': [u'string', u'null']},
                u'sha1': {u'type': [u'string', u'null']},
                u'sha256': {u'type': [u'string', u'null']},
                u'content_type': {
                    u'type': u'string'},
                u'external': {
                    u'type': u'boolean'},
                u'size': {u'type': [
                    u'number',
                    u'null']},
                u'status': {
                    u'enum': [
                        u'saving',
                        u'active',
                        u'pending_delete'],
                    u'type': u'string'}},
            u'required': [u'size',
                          u'md5', u'sha1', u'sha256',
                          u'external',
                          u'status',
                          u'content_type'],
            u'required_on_activate': False,
            u'type': [u'object',
                      u'null']},
        u'template': {
            u'additionalProperties': False,
            u'description': u'TOSCA template body.',
            u'filter_ops': [],
            u'glareType': u'Blob',
            u'properties': {
                u'md5': {u'type': [u'string', u'null']},
                u'sha1': {u'type': [u'string', u'null']},
                u'sha256': {u'type': [u'string', u'null']},
                u'content_type': {
                    u'type': u'string'},
                u'external': {u'type': u'boolean'},
                u'size': {u'type': [u'number',
                                    u'null']},
                u'status': {u'enum': [u'saving',
                                      u'active',
                                      u'pending_delete'],
                            u'type': u'string'}},
            u'required': [u'size',
                          u'md5', u'sha1', u'sha256',
                          u'external',
                          u'status',
                          u'content_type'],
            u'type': [u'object',
                      u'null']},
    }
}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib import utils

from glareclient._i18n import _

LOG = logging.getLogger(__name__)

DEFAULT_API_VERSION = "1"
API_VERSION_OPTION = "os_artifact_api_version"
API_NAME = "artifact"
API_VERSIONS = {
    '1': 'glareclient.v1.client.Client',
}


def make_client(instance):
    """Returns an artifact service client"""
    glare_client = utils.get_client_class(
        API_NAME,
        instance._api_version[API_NAME],
        API_VERSIONS)
    LOG.debug("Instantiating glare client: {0}".format(
              glare_client))
    kwargs = dict(
        region_name=instance._region_name,
        session=instance.session,
        service_type='artifact',
    )
    return glare_client(instance.get_configuration().get('glare_url'),
                        **kwargs)


def build_option_parser(parser):
    """Hook to add global options"""
    parser.add_argument(
        '--os-artifact-api-version',
        metavar='<artifact-api-version>',
        default=utils.env('OS_ARTIFACT_API_VERSION'),
        help=_('Artifact API version, default=%s '
               '(Env: OS_ARTIFACT_API_VERSION)') % DEFAULT_API_VERSION)

    return parser



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from oslo_serialization import jsonutils
from oslo_utils import encodeutils
import six
from six.moves.urllib import parse

from glareclient.common import utils
from glareclient import exc


class Controller(object):
    def __init__(self, http_client, type_name=None):
        self.http_client = http_client
        self.type_name = type_name
        self.default_page_size = 20
        self.sort_dir_values = ('asc', 'desc')

    def _check_type_name(self, type_name):
        """Check that type name and type versions were specified."""
        type_name = type_name or self.type_name
        if type_name is None:
            msg = "Type name must be specified"
            raise exc.HTTPBadRequest(msg)
        return type_name

    def _validate_sort_param(self, sort):
        """Validates sorting argument for invalid keys and directions values.

        :param sort: comma-separated list of sort keys with optional <:dir>
        after each key
        """
        for sort_param in sort.strip().split(','):
            key, _sep, dir = sort_param.partition(':')
            if dir and dir not in self.sort_dir_values:
                msg = ('Invalid sort direction: %(sort_dir)s.'
                       ' It must be one of the following: %(available)s.'
                       ) % {'sort_dir': dir,
                            'available': ', '.join(self.sort_dir_values)}
                raise exc.HTTPBadRequest(msg)
        return sort

    def create(self, name, version='0.0.0', type_name=None, **kwargs):
        """Create an artifact of given type and version.

        :param name: name of creating artifact.
        :param version: semver string describing an artifact version
        """
        type_name = self._check_type_name(type_name)
        kwargs.update({'name': name, 'version': version})
        url = '/artifacts/%s' % type_name
        resp, body = self.http_client.post(url, json=kwargs)
        return body

    def update(self, artifact_id, type_name=None, remove_props=None,
               **kwargs):
        """Update attributes of an artifact.

        :param artifact_id: ID of the artifact to modify.
        :param remove_props: List of property names to remove
        :param \*\*kwargs: Artifact attribute names and their new values.
        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s/%s' % (type_name, artifact_id)
        hdrs = {'Content-Type': 'application/json-patch+json'}
        changes = []
        if remove_props:
            for prop_name in remove_props:
                if prop_name not in kwargs:
                    if '/' in prop_name:
                        # we remove all values in dicts and lists explicitly,
                        # i.e. matadata/key or releases/1
                        changes.append({'op': 'remove',
                                        'path': '/%s' % prop_name})
                    else:
                        # in other cases we just replace the value with None
                        changes.append({'op': 'replace',
                                        'path': '/%s' % prop_name,
                                        'value': None})
        for prop_name in kwargs:
            changes.append({'op': 'add', 'path': '/%s' % prop_name,
                            'value': kwargs[prop_name]})
        resp, body = self.http_client.patch(url, headers=hdrs, json=changes)
        return body

    def get(self, artifact_id, type_name=None):
        """Get information about an artifact.

        :param artifact_id: ID of the artifact to get.

        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s/%s' % (type_name, artifact_id)
        resp, body = self.http_client.get(url)
        return body

    def get_by_name(self, name, version='latest', type_name=None):
        """Get information about an artifact by name.

        :param name: name of the artifact to get.
        :param version: version of the artifact to get
        :param type_name: type name of the artifact
        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s?version=%s&name=%s' % (type_name, version, name)
        resp, body = self.http_client.get(url)
        arts = body.get('artifacts', body.get(type_name))
        if not arts:
            msg = ('Artifact with name=%s and version=%s not found.' %
                   (name, version))
            raise exc.BadRequest(msg)
        if len(arts) > 1:
            if type_name != "all":
                output = "\n".join([
                    "Artifact: %s, owner: %s, visibility: %s" % (
                        i['id'], i['owner'], i['visibility']) for i in arts])
            else:
                output = "\n".join([
                    "Artifact: %s, owner: %s, visibility: %s, type: %s" % (
                        i['id'], i['owner'], i['visibility'], i['type_name'])
                    for i in arts])
            msg = (
                'There are more then one artifact with name=%s and version=%s.'
                ' Please provide the concrete id from the list:\n%s' %
                (name, version, output))
            raise exc.BadRequest(msg)
        return arts[0]

    def list(self, type_name=None, **kwargs):
        """Retrieve a listing of artifacts objects.

        :param page_size: Number of artifacts to request in each
                          paginated request.
        :returns: generator over list of artifacts.
        """
        type_name = self._check_type_name(type_name)

        limit = kwargs.get('limit')
        page_size = kwargs.get('page_size') or self.default_page_size

        def paginate(url, page_size, limit=None):
            next_url = url

            while True:
                if limit and page_size > limit:
                    next_url = next_url.replace("limit=%s" % page_size,
                                                "limit=%s" % limit)

                resp, body = self.http_client.get(next_url)

                # For backward compatibility we also look for the list of
                # artifacts under the type_name section.
                # In current versions it should be located in 'artifacts'.
                for artifact in body.get('artifacts', body.get(type_name)):
                    yield artifact

                    if limit:
                        limit -= 1
                        if limit <= 0:
                            raise StopIteration

                try:
                    next_url = body['next']
                except KeyError:
                    return

        filters = kwargs.get('filters', [])
        filters.append(('limit', page_size))
        if kwargs.get('marker'):
            filters.append(('marker', kwargs.get('marker')))

        url_params = []
        for param, items in filters:
            values = [items] if not isinstance(items, list) else items
            for value in values:
                if isinstance(value, six.string_types):
                    value = encodeutils.safe_encode(value)
                url_params.append({param: value})

        url = '/artifacts/%s?' % type_name

        for param in url_params:
            url = '%s&%s' % (url, parse.urlencode(param))

        if 'sort' in kwargs:
            url = '%s&sort=%s' % (url, self._validate_sort_param(
                kwargs['sort']))

        for artifact in paginate(url, page_size, limit):
            yield artifact

    def activate(self, artifact_id, type_name=None):
        """Set artifact status to 'active'.

        :param artifact_id: ID of the artifact to get.
        """
        return self.update(artifact_id, type_name,
                           status='active')

    def deactivate(self, artifact_id, type_name=None):
        """Set artifact status to 'deactivated'.

        :param artifact_id: ID of the artifact to get.
        """
        return self.update(artifact_id, type_name,
                           status='deactivated')

    def reactivate(self, artifact_id, type_name=None):
        """Set artifact status to 'active'.

        :param artifact_id: ID of the artifact to get.
        """
        return self.update(artifact_id, type_name,
                           status='active')

    def publish(self, artifact_id, type_name=None):
        """Set artifact visibility to 'public'.

        :param artifact_id: ID of the artifact to get.
        """
        return self.update(artifact_id, type_name,
                           visibility='public')

    def delete(self, artifact_id, type_name=None):
        """Delete an artifact and all its data.

        :param artifact_id: ID of the artifact to delete.
        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s/%s' % (type_name, artifact_id)
        self.http_client.delete(url)

    def upload_blob(self, artifact_id, blob_property, data, type_name=None,
                    content_type=None):
        """Upload blob data.

        :param artifact_id: ID of the artifact to download a blob
        :param blob_property: blob property name
        """
        content_type = content_type or 'application/octet-stream'
        hdrs = {'Content-Type': content_type}
        type_name = self._check_type_name(type_name)

        content_length = None
        if isinstance(data, six.string_types):
            content_length = len(data)
        else:
            try:
                content_length = os.path.getsize(data.name)
            except Exception:
                # if for some reason we can't get the file size, then we just
                # ignore it.
                pass
        if content_length is not None:
            hdrs['Content-Length'] = str(content_length)

        url = '/artifacts/%s/%s/%s' % (type_name, artifact_id, blob_property)
        self.http_client.put(url, headers=hdrs, data=data)

    def add_external_location(self, artifact_id, blob_property, data,
                              type_name=None):
        """Add external location.

        :param artifact_id: ID of the artifact to download a blob
        :param blob_property: blob property name
        """
        content_type = 'application/vnd+openstack.glare-custom-location+json'

        type_name = self._check_type_name(type_name)
        hdrs = {'Content-Type': content_type}
        url = '/artifacts/%s/%s/%s' % (type_name, artifact_id, blob_property)
        try:
            data = jsonutils.dumps(data)
        except TypeError:
            raise exc.HTTPBadRequest("json is malformed.")
        self.http_client.put(url, headers=hdrs, data=data)

    def remove_external_location(self, artifact_id, blob_property,
                                 type_name=None):
        """Remove external location.

        :param artifact_id: ID of the artifact with external location
         to be removed
        :param blob_property: blob property name
        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s/%s/%s' % (type_name, artifact_id, blob_property)
        self.http_client.delete(url)

    def download_blob(self, artifact_id, blob_property, type_name=None,
                      do_checksum=True):
        """Get blob data.

        :param artifact_id: ID of the artifact to download a blob
        :param blob_property: blob property name
        :param do_checksum: Enable/disable checksum validation.
        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s/%s/%s' % (type_name, artifact_id, blob_property)
        resp, body = self.http_client.get(url, redirect=False,
                                          stream=True,
                                          headers={"Accept": "*/*"})
        return utils.ResponseBlobWrapper(resp, do_checksum)

    def get_type_list(self):
        """Get list of type names."""
        resp, body = self.http_client.get('/schemas')
        type_list = []
        for type_name, type_schema in six.iteritems(body['schemas']):
            type_list.append((type_name, type_schema['version']))
        return type_list

    def get_type_schema(self, type_name=None):
        """Show schema of type name."""

        type_name = self._check_type_name(type_name)
        url = '/schemas/%s' % type_name
        resp, body = self.http_client.get(url)
        return body['schemas'][type_name]

    def add_tag(self, artifact_id, tag_value, type_name=None):
        """Add tag to artifact.

        :param artifact_id: ID of the artifact to add a tag
        :param tag_value: value of the tag to add
        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s/%s' % (type_name, artifact_id)
        resp, body = self.http_client.get(url)
        tags = body['tags']
        if tag_value in tags:
            return body
        tags.append(tag_value)
        return self.update(artifact_id, type_name, tags=tags)

    def remove_tag(self, artifact_id, tag_value, type_name=None):
        """Remove tag from artifact.

        :param artifact_id: ID of the artifact to remove a tag
        :param tag_value: value of the tag to remove
        """
        type_name = self._check_type_name(type_name)
        url = '/artifacts/%s/%s' % (type_name, artifact_id)
        resp, body = self.http_client.get(url)
        tags = body['tags']
        if tag_value not in tags:
            return body
        tags.remove(tag_value)
        return self.update(artifact_id, type_name, tags=tags)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

from osc_lib.command import command

from glareclient.common import progressbar
from glareclient.common import utils
from glareclient import exc
from glareclient.osc.v1 import TypeMapperAction

LOG = logging.getLogger(__name__)


def _default_blob_property(type_name):
    if type_name == 'images':
        return 'image'
    elif type_name == 'murano_packages':
        return 'package'
    elif type_name in ('heat_templates', 'tosca_templates'):
        return 'template'
    elif type_name == 'heat_environments':
        return 'environment'
    utils.exit('Unknown artifact type. Please specify --blob-property.')


class UploadBlob(command.ShowOne):
    """Upload blob"""

    def get_parser(self, prog_name):
        parser = super(UploadBlob, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to upload.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='The specified id of the artifact.',
        ),
        parser.add_argument(
            '--file',
            metavar='<FILE_PATH>',
            help='Local file that contains data to be uploaded.',
        )
        parser.add_argument(
            '--blob-property', '-p',
            metavar='<BLOB_PROPERTY>',
            help='Name of the blob field.'
        )
        parser.add_argument(
            '--content-type', '-C',
            metavar='<CONTENT_TYPE>',
            default='application/octet-stream',
            help='Content-type of the blob.'
        )
        parser.add_argument(
            '--progress',
            action='store_true',
            help='Show download progress bar.'
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = utils.get_artifact_id(client, parsed_args)

        if not parsed_args.blob_property:
            parsed_args.blob_property = _default_blob_property(
                parsed_args.type_name)

        if parsed_args.file is None:
            if sys.stdin.isatty():
                raise exc.CommandError('Blob file should be specified or '
                                       'explicitly connected to stdin')
            blob = sys.stdin
        else:
            blob = open(parsed_args.file, 'rb')
        if parsed_args.progress:
            blob = progressbar.VerboseFileWrapper(blob)

        client.artifacts.upload_blob(af_id, parsed_args.blob_property, blob,
                                     content_type=parsed_args.content_type,
                                     type_name=parsed_args.type_name)

        data = client.artifacts.get(af_id, type_name=parsed_args.type_name)

        data_to_display = {'blob_property': parsed_args.blob_property}
        if '/' in parsed_args.blob_property:
            dict_name, __, key_name = parsed_args.blob_property.partition('/')
            data_to_display.update(data[dict_name][key_name])
        else:
            data_to_display.update(data[parsed_args.blob_property])
        return self.dict2columns(data_to_display)


class DownloadBlob(command.Command):
    """Download blob"""

    def get_parser(self, prog_name):
        parser = super(DownloadBlob, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to download.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='The specified id of the artifact.',
        ),
        parser.add_argument(
            '--progress',
            action='store_true',
            help='Show download progress bar.'
        )
        parser.add_argument(
            '--file',
            metavar='<FILE>',
            help='Local file to save downloaded blob to. '
                 'If this is not specified and there is no redirection '
                 'the blob will not be saved.'
        )
        parser.add_argument(
            '--blob-property', '-p',
            metavar='<BLOB_PROPERTY>',
            default=None,
            help='Name of the blob field.'
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        if not parsed_args.blob_property:
            parsed_args.blob_property = _default_blob_property(
                parsed_args.type_name)
        af_id = utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.download_blob(af_id,
                                              parsed_args.blob_property,
                                              type_name=parsed_args.type_name)
        if parsed_args.progress:
            data = progressbar.VerboseFileWrapper(data)
        if not (sys.stdout.isatty() and parsed_args.file is None):
            utils.save_blob(data, parsed_args.file)
        else:
            msg = ('No redirection or local file specified for downloaded '
                   'blob. Please specify a local file with --file to save '
                   'downloaded blob or redirect output to another source.')
            utils.exit(msg)


class AddLocation(command.ShowOne):
    """Add external location"""

    def get_parser(self, prog_name):
        parser = super(AddLocation, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to download.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='The specified id of the artifact.',
        ),
        parser.add_argument(
            '--url',
            metavar='<FILE_PATH>',
            help='External location that contains data to be uploaded.',
        )
        parser.add_argument(
            '--md5',
            metavar='<FILE_PATH>',
            help='Specify a checksum md5.',
        )
        parser.add_argument(
            '--sha1',
            metavar='<FILE_PATH>',
            help='Specify a checksum sha1.',
        )
        parser.add_argument(
            '--sha256',
            metavar='<FILE_PATH>',
            help='Specify a checksum sha256.',
        )
        parser.add_argument(
            '--blob-property', '-p',
            metavar='<BLOB_PROPERTY>',
            help='Name of the blob field.'
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = utils.get_artifact_id(client, parsed_args)

        if not parsed_args.blob_property:
            parsed_args.blob_property = _default_blob_property(
                parsed_args.type_name)

        data = {
            'url': parsed_args.url,
            'md5': parsed_args.md5,
            'sha1': parsed_args.sha1,
            'sha256': parsed_args.sha256
        }

        client.artifacts.add_external_location(
            af_id,
            parsed_args.blob_property,
            data,
            type_name=parsed_args.type_name)

        data = client.artifacts.get(af_id,
                                    type_name=parsed_args.type_name)

        data_to_display = {'blob_property': parsed_args.blob_property}
        if '/' in parsed_args.blob_property:
            dict_name, __, key_name = parsed_args.blob_property.partition('/')
            data_to_display.update(data[dict_name][key_name])
        else:
            data_to_display.update(data[parsed_args.blob_property])
        return self.dict2columns(data_to_display)


class RemoveLocation(command.ShowOne):
    """Remove external location"""

    def get_parser(self, prog_name):
        parser = super(RemoveLocation, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to download.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='The specified id of the artifact.',
        ),
        parser.add_argument(
            '--blob-property', '-p',
            metavar='<BLOB_PROPERTY>',
            help='Name of the blob field.'
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = utils.get_artifact_id(client, parsed_args)

        if not parsed_args.blob_property:
            parsed_args.blob_property = _default_blob_property(
                parsed_args.type_name)

        client.artifacts.remove_external_location(
            af_id,
            parsed_args.blob_property,
            type_name=parsed_args.type_name)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import print_function

import errno
import hashlib
import os
import sys

if os.name == 'nt':
    import msvcrt
else:
    msvcrt = None

from oslo_log import log as logging
from oslo_utils import encodeutils
from oslo_utils import importutils
import requests

from glareclient import exc

LOG = logging.getLogger(__name__)


SENSITIVE_HEADERS = ('X-Auth-Token', )


def env(*vars, **kwargs):
    """Search for the first defined of possibly many env vars.

    Returns the first environment variable defined in vars, or
    returns the default defined in kwargs.
    """
    for v in vars:
        value = os.environ.get(v, None)
        if value:
            return value
    return kwargs.get('default', '')


def import_versioned_module(version, submodule=None):
    module = 'glareclient.v%s' % version
    if submodule:
        module = '.'.join((module, submodule))
    return importutils.import_module(module)


def exit(msg='', exit_code=1):
    if msg:
        print(encodeutils.safe_decode(msg), file=sys.stderr)
    sys.exit(exit_code)


class ResponseBlobWrapper(object):
    """Represent HTTP response as iterator with known length."""

    def __init__(self, resp, verify_md5=True):
        self.hash_md5 = resp.headers.get("Content-MD5")
        self.blob_md5 = hashlib.md5()
        if 301 <= resp.status_code <= 302:
            # NOTE(sskripnick): handle redirect manually to prevent sending
            # auth token to external resource.
            # Use stream=True to prevent reading whole response into memory.
            # Set Accept-Encoding explicitly to "identity" because setting
            # stream=True forces Accept-Encoding to be "gzip, deflate".
            # It should be "identity" because we should know Content-Length.
            resp = requests.get(resp.headers.get("Location"),
                                headers={"Accept-Encoding": "identity"})
        self.len = resp.headers.get("Content-Length", 0)
        self.iter = resp.iter_content(65536)
        self.verify_md5 = verify_md5

    def __iter__(self):
        return self

    def next(self):
        try:
            data = self.iter.next()
            if self.verify_md5:
                self.blob_md5.update(data)
            return data
        except StopIteration:
            if self.verify_md5 and self.blob_md5.hexdigest() != self.hash_md5:
                raise IOError(errno.EPIPE,
                              'Checksum mismatch: %s (expected %s)' %
                              (self.blob_md5.hexdigest(), self.hash_md5))
            raise

    __next__ = next


def get_item_properties(item, fields, mixed_case_fields=None, formatters=None):
    """Return a tuple containing the item properties.

    :param item: a single item resource (e.g. Server, Project, etc)
    :param fields: tuple of strings with the desired field names
    :param mixed_case_fields: tuple of field names to preserve case
    :param formatters: dictionary mapping field names to callables
       to format the values
    """
    if mixed_case_fields is None:
        mixed_case_fields = []
    if formatters is None:
        formatters = {}

    row = []

    for field in fields:
        if field in mixed_case_fields:
            field_name = field.replace(' ', '_')
        else:
            field_name = field.lower().replace(' ', '_')
        data = item[field_name]
        if field in formatters:
            row.append(formatters[field](data))
        else:
            row.append(data)
    return tuple(row)


def make_size_human_readable(size):
    suffix = ['B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB']
    base = 1024.0
    index = 0

    if size is None:
        size = 0
    while size >= base:
        index = index + 1
        size = size / base

    padded = '%.1f' % size
    stripped = padded.rstrip('0').rstrip('.')

    return '%s%s' % (stripped, suffix[index])


def save_blob(data, path):
    """Save a blob to the specified path.

    :param data: blob of the artifact
    :param path: path to save the blob to
    """
    if path is None:
        blob = getattr(sys.stdout, 'buffer',
                       sys.stdout)
    else:
        blob = open(path, 'wb')
    try:
        for chunk in data:
            blob.write(chunk)
    finally:
        if path is not None:
            blob.close()


def get_artifact_id(client, parsed_args):
    if parsed_args.id:
        return parsed_args.name
    try:
        return client.artifacts.get_by_name(
            parsed_args.name,
            version=parsed_args.artifact_version,
            type_name=parsed_args.type_name)['id']
    except exc.BadRequest as e:
        exit(msg=e.details)


def get_system_ca_file():
    """Return path to system default CA file."""
    # Standard CA file locations for Debian/Ubuntu, RedHat/Fedora,
    # Suse, FreeBSD/OpenBSD, MacOSX, and the bundled ca
    ca_path = ['/etc/ssl/certs/ca-certificates.crt',
               '/etc/pki/tls/certs/ca-bundle.crt',
               '/etc/ssl/ca-bundle.pem',
               '/etc/ssl/cert.pem',
               '/System/Library/OpenSSL/certs/cacert.pem',
               requests.certs.where()]
    for ca in ca_path:
        LOG.debug("Looking for ca file %s", ca)
        if os.path.exists(ca):
            LOG.debug("Using ca file %s", ca)
            return ca
    LOG.warning("System ca file could not be found.")



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

import mock
from osc_lib.tests import utils

from glareclient.common import utils as g_utils
from glareclient.tests.unit.osc.v1 import fakes_schemas

blob_fixture = {
    "status": "active",
    "url": "fake_url",
    "md5": "35d83e8eedfbdb87ff97d1f2761f8ebf",
    "sha1": "942854360eeec1335537702399c5aed940401602",
    "sha256": "d8a7834fc6652f316322d80196f6dcf2"
              "94417030e37c15412e4deb7a67a367dd",
    "external": False,
    "content_type": "application/octet-stream",
    "size": 594}


def mock_list(*args, **kwargs):
    return [{'id': 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
             'name': 'art1',
             'version': '0.0.0',
             'owner': 'f649c77999e449e89627024f71b76603',
             'visibility': 'private',
             'status': 'active',
             'type_name': 'images'},
            {'id': '48d35c1d-6739-459b-bbda-e4dcba8a684a',
             'name': 'art2',
             'version': '0.0.0',
             'owner': 'f649c77999e449e89627024f71b76603',
             'visibility': 'private',
             'status': 'active',
             'type_name': 'heat_templates'}]


def mock_get(*args, **kwargs):
    return {'id': 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
            'name': 'art1',
            'version': '0.0.0',
            'owner': 'f649c77999e449e89627024f71b76603',
            'visibility': 'private',
            'status': 'active',
            'blob': blob_fixture,
            'image': blob_fixture,
            'package': blob_fixture,
            'template': blob_fixture,
            'environment': blob_fixture}


def mock_g_servs(*args, **kwargs):
    return {'id': 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
            'name': 'art1',
            'version': '0.0.0',
            'owner': 'f649c77999e449e89627024f71b76603',
            'visibility': 'private',
            'status': 'active'}


def mock_g_schema(*args, **kwargs):
    return fakes_schemas.FIXTURE_SCHEMA


def mock_get_data_file(*args, **kwargs):
    return 'data'


class TestArtifacts(utils.TestCommand):

    def setUp(self):
        super(TestArtifacts, self).setUp()
        self.app.client_manager.artifact = mock.MagicMock()
        self.app.client_manager.artifact.artifacts.list = mock_list
        self.app.client_manager.artifact.artifacts.get = mock_get
        self.app.client_manager.artifact.artifacts.get_by_name = mock_get
        self.app.client_manager.artifact.artifacts.add_tag = mock_g_servs
        self.app.client_manager.artifact.artifacts.remove_tag = mock_g_servs
        self.app.client_manager.artifact.artifacts.create = mock_g_servs
        self.app.client_manager.artifact.artifacts.update = mock_g_servs
        self.app.client_manager.artifact.artifacts.delete = mock_g_servs
        self.app.client_manager.artifact.artifacts.activate = mock_g_servs
        self.app.client_manager.artifact.artifacts.deactivate = mock_g_servs
        self.app.client_manager.artifact.artifacts.reactivate = mock_g_servs
        self.app.client_manager.artifact.artifacts.publish = mock_g_servs
        self.app.client_manager.artifact.blobs.upload_blob = mock_g_servs
        self.app.client_manager.artifact.blobs.download_blob = mock_g_servs
        self.app.client_manager.artifact.blobs.add_external_location = \
            mock_g_servs
        self.app.client_manager.artifact.artifacts.get_type_schema = \
            mock_g_schema
        g_utils.get_data_file = mock.MagicMock()
        g_utils.get_data_file = mock_get_data_file
        g_utils.save_blob = mock.MagicMock()
        sys.stdout.isatty = mock.MagicMock()
        sys.stdout.isatty._mock_return_value = True



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io
import os
import tempfile

import mock
from oslo_serialization import jsonutils
import testtools

from glareclient.v1 import artifacts


class TestController(testtools.TestCase):

    def setUp(self):
        self.mock_resp = mock.MagicMock()
        self.mock_body = mock.MagicMock()
        self.mock_http_client = mock.MagicMock()
        for method in ('get', 'post', 'patch', 'delete'):
            method = getattr(self.mock_http_client, method)
            method.return_value = (self.mock_resp, self.mock_body)
        self.c = artifacts.Controller(self.mock_http_client, 'test_name')
        self.c._check_type_name = mock.Mock(return_value='checked_name')
        super(TestController, self).setUp()

    def test_create(self):
        body = self.c.create('name', version='0.1.2', type_name='ok')
        self.assertEqual(self.mock_body, body)
        self.mock_http_client.post.assert_called_once_with(
            '/artifacts/checked_name',
            json={'version': '0.1.2', 'name': 'name'})
        self.c._check_type_name.assert_called_once_with('ok')

    def test_update(self):
        remove_props = ['remove1', 'remove2']
        body = self.c.update('test-id', type_name='test_name',
                             remove_props=remove_props, update1=1, update2=2)
        self.assertEqual(self.mock_body, body)
        patch_kwargs = {
            'headers': {'Content-Type': 'application/json-patch+json'},
            'json': [
                {'path': '/remove1', 'value': None, 'op': 'replace'},
                {'path': '/remove2', 'value': None, 'op': 'replace'},
                {'path': '/update2', 'value': 2, 'op': 'add'},
                {'path': '/update1', 'value': 1, 'op': 'add'}
            ]
        }
        self.mock_http_client.patch.assert_called_once_with(
            '/artifacts/checked_name/test-id', **patch_kwargs)
        self.c._check_type_name.assert_called_once_with('test_name')

    def test_get(self):
        body = self.c.get('test-id', type_name='test_name')
        self.assertEqual(self.mock_body, body)
        self.mock_http_client.get.assert_called_once_with(
            '/artifacts/checked_name/test-id')
        self.c._check_type_name.assert_called_once_with('test_name')

    def test_list_legacy(self):
        self.mock_http_client.get.side_effect = [
            (None, {'checked_name': [10, 11, 12], "next": "next1"}),
            (None, {'checked_name': [13, 14, 15], "next": "next2"}),
            (None, {'checked_name': [16, 17, 18], "next": "next3"}),
            (None, {'checked_name': [19, 20, 21]}),
        ]
        data = list(self.c.list(type_name='test-type', limit=10, page_size=3))
        expected = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
        self.assertEqual(expected, data)
        expected_calls = [
            mock.call.get('/artifacts/checked_name?&limit=3'),
            mock.call.get('next1'),
            mock.call.get('next2'),
            mock.call.get('next3'),
        ]
        self.assertEqual(expected_calls, self.mock_http_client.mock_calls)

    def test_list(self):
        self.mock_http_client.get.side_effect = [
            (None, {'artifacts': [10, 11, 12], "next": "next1"}),
            (None, {'artifacts': [13, 14, 15], "next": "next2"}),
            (None, {'artifacts': [16, 17, 18], "next": "next3"}),
            (None, {'artifacts': [19, 20, 21]}),
        ]
        data = list(self.c.list(type_name='test-type', limit=10, page_size=3))
        expected = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
        self.assertEqual(expected, data)
        expected_calls = [
            mock.call.get('/artifacts/checked_name?&limit=3'),
            mock.call.get('next1'),
            mock.call.get('next2'),
            mock.call.get('next3'),
        ]
        self.assertEqual(expected_calls, self.mock_http_client.mock_calls)

    def test_activate(self):
        self.c.update = mock.Mock()
        self.assertEqual(self.c.update.return_value,
                         self.c.activate('test-id', type_name='test-type'))
        self.c.update.assert_called_once_with('test-id', 'test-type',
                                              status='active')

    def test_deactivate(self):
        self.c.update = mock.Mock()
        self.assertEqual(self.c.update.return_value,
                         self.c.deactivate('test-id', type_name='test-type'))
        self.c.update.assert_called_once_with('test-id', 'test-type',
                                              status='deactivated')

    def test_reactivate(self):
        self.c.update = mock.Mock()
        self.assertEqual(self.c.update.return_value,
                         self.c.reactivate('test-id', type_name='test-type'))
        self.c.update.assert_called_once_with('test-id', 'test-type',
                                              status='active')

    def test_delete(self):
        self.assertIsNone(self.c.delete('test-id', type_name='test-name'))
        self.mock_http_client.delete.assert_called_once_with(
            '/artifacts/checked_name/test-id')

    def test_upload_blob_str(self):
        self.c.upload_blob('test-id', 'blob-prop', 'data',
                           type_name='test-type',
                           content_type='application/test')
        self.mock_http_client.put.assert_called_once_with(
            '/artifacts/checked_name/test-id/blob-prop',
            data='data', headers={'Content-Length': '4',
                                  'Content-Type': 'application/test'})

    def test_upload_blob_file(self):
        tfd, path = tempfile.mkstemp()
        try:
            os.write(tfd, b'data')
            tfile = open(path, "rb")
            self.c.upload_blob('test-id', 'blob-prop', tfile,
                               type_name='test-type',
                               content_type='application/test')
            self.mock_http_client.put.assert_called_once_with(
                '/artifacts/checked_name/test-id/blob-prop',
                data=tfile, headers={'Content-Length': '4',
                                     'Content-Type': 'application/test'})
        finally:
            os.remove(path)

    def test_upload_blob_stream(self):
        data = io.BytesIO(b'data')
        self.c.upload_blob('test-id', 'blob-prop', data,
                           type_name='test-type',
                           content_type='application/test')
        self.mock_http_client.put.assert_called_once_with(
            '/artifacts/checked_name/test-id/blob-prop',
            data=data, headers={'Content-Type': 'application/test'})

    def test_get_type_list(self):
        schemas = {'schemas': {'a': {'version': 1}, 'b': {'version': 2}}}
        self.mock_http_client.get.return_value = (None, schemas)
        expected_types = [('a', 1), ('b', 2)]
        self.assertEqual(expected_types, self.c.get_type_list())

    def test_get_type_schema(self):
        test_schema = {'schemas': {'checked_name': 'test-schema'}}
        self.mock_http_client.get.return_value = (None, test_schema)
        self.assertEqual('test-schema',
                         self.c.get_type_schema(type_name='test-type'))
        self.mock_http_client.get.assert_called_once_with(
            '/schemas/checked_name')

    def test_add_external_location(self):
        art_id = '3a4560a1-e585-443e-9b39-553b46ec92a8'
        data = {
            'url': 'http://fake_url',
            'md5': '7CA772EE98D5CAF99F3674085D5E4124',
            'sha1': None,
            'sha256': None},
        resp = self.c.add_external_location(
            art_id, 'image',
            data=data,
            type_name='images')
        self.c.http_client.put.assert_called_once_with(
            '/artifacts/checked_name/'
            '3a4560a1-e585-443e-9b39-553b46ec92a8/image',
            data=jsonutils.dumps(data),
            headers={'Content-Type':
                     'application/vnd+openstack.glare-custom-location+json'})
        self.assertIsNone(resp)

    def test_remove_external_location(self):
        art_id = '3a4560a1-e585-443e-9b39-553b46ec92a8'
        resp = self.c.remove_external_location(
            art_id, 'image',
            type_name='images')
        self.c.http_client.delete.assert_called_once_with(
            '/artifacts/checked_name/'
            '3a4560a1-e585-443e-9b39-553b46ec92a8/image')
        self.assertIsNone(resp)

    def test_add_tag(self):
        art_id = '07a679d8-d0a8-45ff-8d6e-2f32f2097b7c'
        d = {'tags': ['a', 'b', 'c']}
        self.mock_body.__getitem__.side_effect = d.__getitem__
        data = self.c.add_tag(
            art_id, tag_value="123", type_name='images')
        self.c.http_client.get.assert_called_once_with(
            '/artifacts/checked_name/07a679d8-d0a8-45ff-8d6e-2f32f2097b7c')
        self.c.http_client.patch.assert_called_once_with(
            '/artifacts/checked_name/07a679d8-d0a8-45ff-8d6e-2f32f2097b7c',
            headers={'Content-Type': 'application/json-patch+json'},
            json=[{'path': '/tags',
                   'value': ['a', 'b', 'c', '123'],
                   'op': 'add'}])
        self.assertIsNotNone(data)

    def test_add_existing_tag(self):
        art_id = '07a679d8-d0a8-45ff-8d6e-2f32f2097b7c'
        d = {'tags': ['a', 'b', 'c']}
        self.mock_body.__getitem__.side_effect = d.__getitem__
        data = self.c.add_tag(
            art_id, tag_value="a", type_name='images')
        self.c.http_client.get.assert_called_once_with(
            '/artifacts/checked_name/07a679d8-d0a8-45ff-8d6e-2f32f2097b7c')
        self.assertEqual(0, self.c.http_client.patch.call_count)
        self.assertIsNotNone(data)

    def test_remove_tag(self):
        art_id = '07a679d8-d0a8-45ff-8d6e-2f32f2097b7c'
        d = {'tags': ['a', 'b', 'c']}
        self.mock_body.__getitem__.side_effect = d.__getitem__
        data = self.c.remove_tag(
            art_id, tag_value="a", type_name='images')
        self.c.http_client.get.assert_called_once_with(
            '/artifacts/checked_name/07a679d8-d0a8-45ff-8d6e-2f32f2097b7c')
        self.c.http_client.patch.assert_called_once_with(
            '/artifacts/checked_name/07a679d8-d0a8-45ff-8d6e-2f32f2097b7c',
            headers={'Content-Type': 'application/json-patch+json'},
            json=[{'path': '/tags',
                   'value': ['b', 'c'],
                   'op': 'add'}])
        self.assertIsNotNone(data)

    def test_remove_nonexisting_tag(self):
        art_id = '07a679d8-d0a8-45ff-8d6e-2f32f2097b7c'
        d = {'tags': ['a', 'b', 'c']}
        self.mock_body.__getitem__.side_effect = d.__getitem__
        data = self.c.remove_tag(
            art_id, tag_value="123", type_name='images')
        self.c.http_client.get.assert_called_once_with(
            '/artifacts/checked_name/07a679d8-d0a8-45ff-8d6e-2f32f2097b7c')
        self.assertEqual(0, self.c.http_client.patch.call_count)
        self.assertIsNotNone(data)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""
Command-line interface to the Glare APIs
"""

import argparse
import logging
import sys

from cliff import app
from cliff import commandmanager
from osc_lib.command import command

from glareclient import client
from glareclient.common import utils
import glareclient.osc.v1.artifacts
import glareclient.osc.v1.blobs


class OpenStackHelpFormatter(argparse.HelpFormatter):
    def __init__(self, prog, indent_increment=2, max_help_position=32,
                 width=None):
        super(OpenStackHelpFormatter, self).__init__(
            prog,
            indent_increment,
            max_help_position,
            width
        )

    def start_section(self, heading):
        # Title-case the headings.
        heading = '%s%s' % (heading[0].upper(), heading[1:])
        super(OpenStackHelpFormatter, self).start_section(heading)


class HelpAction(argparse.Action):
    """Custom help action.

    Provide a custom action so the -h and --help options
    to the main app will print a list of the commands.
    The commands are determined by checking the CommandManager
    instance, passed in as the "default" value for the action.
    """
    def __call__(self, parser, namespace, values, option_string=None):
        outputs = []
        max_len = 0
        app = self.default
        parser.print_help(app.stdout)
        app.stdout.write('\nCommands for API v1 :\n')

        for name, ep in sorted(app.command_manager):
            factory = ep.load()
            cmd = factory(self, None)
            one_liner = cmd.get_description().split('\n')[0]
            outputs.append((name, one_liner))
            max_len = max(len(name), max_len)

        for (name, one_liner) in outputs:
            app.stdout.write('  %s  %s\n' % (name.ljust(max_len), one_liner))

        sys.exit(0)


class BashCompletionCommand(command.Command):
    """Prints all of the commands and options for bash-completion."""

    def take_action(self, parsed_args):
        commands = set()
        options = set()

        for option, _action in self.app.parser._option_string_actions.items():
            options.add(option)

        for command_name, _cmd in self.app.command_manager:
            commands.add(command_name)

        print(' '.join(commands | options))


class GlareShell(app.App):

    def __init__(self):
        super(GlareShell, self).__init__(
            description=__doc__.strip(),
            version=glareclient.__version__,
            command_manager=commandmanager.CommandManager('glare.cli'),
        )
        self._set_shell_commands(self._get_commands())

    def configure_logging(self):
        log_lvl = logging.DEBUG if self.options.debug else logging.WARNING
        logging.basicConfig(
            format="%(levelname)s (%(module)s) %(message)s",
            level=log_lvl
        )
        logging.getLogger('iso8601').setLevel(logging.WARNING)

        if self.options.verbose_level <= 1:
            logging.getLogger('requests').setLevel(logging.WARNING)

    def build_option_parser(self, description, version,
                            argparse_kwargs=None):
        """Return an argparse option parser for this application.

        Subclasses may override this method to extend
        the parser with more global options.
        :param description: full description of the application
        :paramtype description: str
        :param version: version number for the application
        :paramtype version: str
        :param argparse_kwargs: extra keyword argument passed to the
                                ArgumentParser constructor
        :paramtype extra_kwargs: dict
        """
        argparse_kwargs = argparse_kwargs or {}

        parser = argparse.ArgumentParser(
            description=description,
            add_help=False,
            formatter_class=OpenStackHelpFormatter,
            **argparse_kwargs
        )
        parser.add_argument(
            '--version',
            action='version',
            version='%(prog)s {0}'.format(version),
            help='Show program\'s version number and exit.'
        )
        parser.add_argument(
            '-v', '--verbose',
            action='count',
            dest='verbose_level',
            default=self.DEFAULT_VERBOSE_LEVEL,
            help='Increase verbosity of output. Can be repeated.',
        )
        parser.add_argument(
            '--log-file',
            action='store',
            default=None,
            help='Specify a file to log output. Disabled by default.',
        )
        parser.add_argument(
            '-q', '--quiet',
            action='store_const',
            dest='verbose_level',
            const=0,
            help='Suppress output except warnings and errors.',
        )
        parser.add_argument(
            '-h', '--help',
            action=HelpAction,
            nargs=0,
            default=self,  # tricky
            help="Show this help message and exit.",
        )
        parser.add_argument(
            '--debug',
            default=False,
            action='store_true',
            help='Show tracebacks on errors.',
        )
        parser.add_argument(
            '--os-glare-url',
            action='store',
            dest='glare_url',
            default=utils.env('OS_GLARE_URL'),
            help='Glare API host (Env: OS_GLARE_URL)'
        )
        parser.add_argument(
            '--os-glare-version',
            action='store',
            dest='glare_version',
            default=utils.env('OS_GLARE_VERSION', default='v1'),
            help='Glare API version (default = v1) (Env: '
                 'OS_GLARE_VERSION)'
        )
        parser.add_argument(
            '--keycloak-auth-url',
            action='store',
            dest='keycloak_auth_url',
            default=utils.env('KEYCLOAK_AUTH_URL'),
            help='Keycloak auth url (Env: KEYCLOAK_AUTH_URL)')
        parser.add_argument(
            '--openid-client-id',
            action='store',
            dest='openid_client_id',
            default=utils.env('OPENID_CLIENT_ID') or 'admin-cli',
            help='Client ID (according to OpenID Connect)'
                 ' (Env: OPENID_CLIENT_ID)')
        parser.add_argument(
            '--auth-token',
            action='store',
            dest='auth_token',
            default=utils.env('AUTH_TOKEN'),
            help='Authentication token (Env: AUTH_TOKEN)')
        parser.add_argument(
            '--keycloak-realm-name',
            action='store',
            dest='keycloak_realm_name',
            default=utils.env('KEYCLOAK_REALM_NAME'),
            help='With keycloak glare auth type: Realm name to scope to'
                 ' (Env: KEYCLOAK_REALM_NAME)')
        parser.add_argument(
            '--keycloak-username',
            action='store',
            dest='keycloak_username',
            default=utils.env('KEYCLOAK_USERNAME'),
            help='Keycloak username (Env: KEYCLOAK_USERNAME)')
        parser.add_argument(
            '--keycloak-password',
            action='store',
            dest='keycloak_password',
            default=utils.env('KEYCLOAK_PASSWORD'),
            help='Keycloak user password (Env: KEYCLOAK_PASSWORD)')
        parser.add_argument(
            '--cert',
            action='store',
            dest='cert_file',
            default=utils.env('OS_GLARE_CERT'),
            help='Client Certificate (Env: OS_GLARE_CERT)'
        )
        parser.add_argument(
            '--key',
            action='store',
            dest='key_file',
            default=utils.env('OS_GLARE_KEY'),
            help='Client Key (Env: OS_GLARE_KEY)'
        )
        parser.add_argument(
            '--cacert',
            action='store',
            dest='cacert',
            default=utils.env('OS_GLARE_CACERT'),
            help='Authentication CA Certificate (Env: OS_GLARE_CACERT)'
        )
        parser.add_argument(
            '--insecure',
            action='store_true',
            dest='insecure',
            default=utils.env('GLARECLIENT_INSECURE', default=False),
            help='Disables SSL/TLS certificate verification '
                 '(Env: GLARELCLIENT_INSECURE)'
        )

        return parser

    def initialize_app(self, argv):
        self._clear_shell_commands()
        self._set_shell_commands(self._get_commands())

        # bash-completion and help messages should not require client creation
        need_client = not (
            ('bash-completion' in argv) or
            ('help' in argv) or
            ('-h' in argv) or
            ('--help' in argv) or
            not argv)

        self.client = self._create_client() if need_client else None

        # Adding client_manager variable to make glare client work with
        # unified OpenStack client.
        ClientManager = type(
            'ClientManager',
            (object,),
            dict(artifact=self.client)
        )

        self.client_manager = ClientManager()

    def _create_client(self):
        return client.Client(
            endpoint=self.options.glare_url,
            auth_token=self.options.auth_token,
            keycloak_auth_url=self.options.keycloak_auth_url,
            openid_client_id=self.options.openid_client_id,
            keycloak_realm_name=self.options.keycloak_realm_name,
            keycloak_username=self.options.keycloak_username,
            keycloak_password=self.options.keycloak_password,
            cert_file=self.options.cert_file,
            key_file=self.options.key_file,
            cacert=self.options.cacert,
            insecure=self.options.insecure
        )

    def _set_shell_commands(self, cmds_dict):
        for k, v in cmds_dict.items():
            self.command_manager.add_command(k, v)

    def _clear_shell_commands(self):
        exclude_cmds = ['help', 'complete']

        cmds = self.command_manager.commands.copy()
        for k, v in cmds.items():
            if k not in exclude_cmds:
                self.command_manager.commands.pop(k)

    @staticmethod
    def _get_commands():
        return {
            'bash-completion': BashCompletionCommand,
            'list': glareclient.osc.v1.artifacts.ListArtifacts,
            'show': glareclient.osc.v1.artifacts.ShowArtifact,
            'create': glareclient.osc.v1.artifacts.CreateArtifact,
            'delete': glareclient.osc.v1.artifacts.DeleteArtifact,
            'update': glareclient.osc.v1.artifacts.UpdateArtifact,
            'activate': glareclient.osc.v1.artifacts.ActivateArtifact,
            'deactivate': glareclient.osc.v1.artifacts.DeactivateArtifact,
            'reactivate': glareclient.osc.v1.artifacts.ReactivateArtifact,
            'publish': glareclient.osc.v1.artifacts.PublishArtifact,
            'add-tag': glareclient.osc.v1.artifacts.AddTag,
            'remove-tag': glareclient.osc.v1.artifacts.RemoveTag,
            'type-list': glareclient.osc.v1.artifacts.TypeList,
            'schema': glareclient.osc.v1.artifacts.TypeSchema,
            'upload': glareclient.osc.v1.blobs.UploadBlob,
            'download': glareclient.osc.v1.blobs.DownloadBlob,
            'location': glareclient.osc.v1.blobs.AddLocation,
            'remove-location': glareclient.osc.v1.blobs.RemoveLocation
        }


def main(argv=sys.argv[1:]):
    return GlareShell().run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from glareclient.common import utils


def Client(version='1', endpoint=None, session=None, *args, **kwargs):
    """Client for the Glare Artifact Repository.

    Generic client for the Glare Artifact Repository. See version classes
    for specific details.
    :param string version: The version of API to use.
    :param session: A keystoneauth1 session that should be used for transport.
    :type session: keystoneauth1.session.Session
    """

    if endpoint is not None:
        kwargs.setdefault('endpoint_override', endpoint)

    if version is None:
        raise RuntimeError("You must provide a client version")

    module = utils.import_versioned_module(int(version), 'client')
    client_class = getattr(module, 'Client')
    return client_class(endpoint, *args, session=session, **kwargs)



# Copyright 2016 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import logging
import pprint
import requests
from six.moves import urllib

from glareclient.common import utils

LOG = logging.getLogger(__name__)


def authenticate(**kwargs):
    """Performs authentication using Keycloak OpenID Protocol.

    :param auth_url: Base authentication url of KeyCloak server (e.g.
            "https://my.keycloak:8443/auth"
    :param client_id: Client ID (according to OpenID Connect protocol).
    :param realm_name: KeyCloak realm name.
    :param username: User name (Optional, if None then access_token must be
            provided).
    :param password: Password (Optional).
    :param cacert: SSL certificate file (Optional).
    :param insecure: If True, SSL certificate is not verified (Optional).
    """
    auth_url = kwargs.get('auth_url')
    client_id = kwargs.get('client_id')
    realm_name = kwargs.get('realm_name')
    username = kwargs.get('username')
    password = kwargs.get('password')
    insecure = kwargs.get('insecure', False)
    cacert = kwargs.get('cacert', utils.get_system_ca_file())

    if not auth_url:
        raise ValueError('Base authentication url is not provided.')

    if not client_id:
        raise ValueError('Client ID is not provided.')

    if not realm_name:
        raise ValueError('Project(realm) name is not provided.')

    if not username:
        raise ValueError('Username is not provided.')

    if password is None:
        raise ValueError('Password is not provided.')

    access_token_endpoint = (
        "%s/realms/%s/protocol/openid-connect/token" %
        (auth_url, realm_name)
    )

    verify = None
    if urllib.parse.urlparse(access_token_endpoint).scheme == "https":
        verify = False if insecure else cacert if cacert else True

    body = {
        'grant_type': 'password',
        'username': username,
        'password': password,
        'scope': 'profile',
        'client_id': client_id
    }

    resp = requests.post(
        access_token_endpoint,
        data=body,
        verify=verify
    )

    try:
        resp.raise_for_status()
    except Exception as e:
        raise Exception("Failed to get access token:\n %s" % str(e))

    LOG.debug(
        "HTTP response from OIDC provider: %s" %
        pprint.pformat(resp.json())
    )

    return resp.json()['access_token']



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse


class TypeMapperAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, self._type_name_mapper(values))

    @staticmethod
    def _type_name_mapper(type_name):
        if type_name in ['images', 'image']:
            return 'images'
        elif type_name in ['heat-templates', 'heat-template',
                           'heat_templates', 'heat_template']:
            return 'heat_templates'
        elif type_name in ['heat-environments', 'heat-environment',
                           'heat_environments', 'heat_environment']:
            return 'heat_environments'
        elif type_name in ['tosca-templates', 'tosca-template',
                           'tosca_templates', 'tosca_template']:
            return 'tosca_templates'
        elif type_name in ['murano-packages', 'murano-package',
                           'murano_packages', 'murano_package']:
            return 'murano_packages'
        return type_name



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib.command import command
from oslo_utils import strutils
from pprint import pformat
import six

from glareclient.common import utils as glare_utils
from glareclient.osc.v1 import TypeMapperAction

LOG = logging.getLogger(__name__)


def print_artifact(client, data, type_name):
        schema = \
            client.artifacts.get_type_schema(type_name=type_name)['properties']

        columns = ('field', 'value', 'glare type')
        column_headers = [c.capitalize() for c in columns]
        table = []

        for key, value in six.iteritems(data):
            if schema[key]['glareType'] == 'Blob':
                value = pformat(value)
            table.append((key, value, schema[key]['glareType']))

        return (column_headers,
                table)


class ListArtifacts(command.Lister):
    """List of artifacts"""

    def get_parser(self, prog_name):
        parser = super(ListArtifacts, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        )
        parser.add_argument(
            '--limit', '-l',
            default=20,
            metavar='<LIMIT>',
            type=int,
            help='Maximum number of artifacts to get.',
        )
        parser.add_argument(
            '--page-size', '-p',
            default=20,
            metavar='<SIZE>',
            type=int,
            help='Number of artifacts to request in each paginated request.',
        )
        parser.add_argument(
            '--filter', '-F',
            default=[],
            action='append',
            metavar='<KEY=VALUE>',
            help='Filtering artifact list by a user-defined property.',
        )
        parser.add_argument(
            '--sort', '-S',
            default='name:asc',
            metavar='<key>[:<direction>]',
            help='Comma-separated list of sort keys and directions in the '
                 'form of <key>[:<asc|desc>].',
        )
        parser.add_argument(
            '--marker', '-m',
            metavar='<MARKER>',
            help="id of last artifact in previous list request"
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        params = {'limit': parsed_args.limit,
                  'filters': [f.split('=', 1) for f in parsed_args.filter],
                  'sort': parsed_args.sort,
                  'page_size': parsed_args.page_size,
                  'marker': parsed_args.marker}

        type_name = parsed_args.type_name

        data = client.artifacts.list(type_name=type_name, **params)

        columns = ['id', 'name', 'version', 'owner', 'visibility', 'status']
        if type_name == 'all':
            columns.insert(3, 'type_name')
        column_headers = [c.capitalize().replace("_", " ") for c in columns]
        table = []
        for af in data:
            table.append(glare_utils.get_item_properties(af, columns))
        return (column_headers,
                table)


class ShowArtifact(command.Lister):
    """Show details artifact"""

    def get_parser(self, prog_name):
        parser = super(ShowArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to show.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        ),
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact

        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.get(af_id,
                                    type_name=parsed_args.type_name)

        return print_artifact(client, data, parsed_args.type_name)


class CreateArtifact(command.Lister):
    """Create a new artifact"""

    def get_parser(self, prog_name):
        parser = super(CreateArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name of the artifact.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            default='0.0.0',
            metavar='<VERSION>',
            help='Version of the artifact.',
        )
        parser.add_argument(
            '--property', '-p',
            metavar='<key=value>',
            action='append',
            default=[],
            help='Simple artifact property.'
        )
        parser.add_argument(
            '--list', '-l',
            metavar='<key=value>',
            action='append',
            default=[],
            help='Artifact list property.'
        )
        parser.add_argument(
            '--dict', '-d',
            metavar='<key=value>',
            action='append',
            default=[],
            help='Artifact dict property.'
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))

        prop = {}
        for datum in parsed_args.property:
            key, value = datum.split('=', 1)
            prop[key] = value

        for datum in parsed_args.list:
            key, value = datum.split('=', 1)
            value = strutils.split_by_commas(value)
            prop[key] = value

        for datum in parsed_args.dict:
            key, value = datum.split('=', 1)
            value = strutils.split_by_commas(value)
            prop[key] = {}
            for elem in value:
                k, v = elem.split(':', 1)
                prop[key][k] = v

        client = self.app.client_manager.artifact
        data = client.artifacts.create(parsed_args.name,
                                       type_name=parsed_args.type_name,
                                       version=parsed_args.artifact_version,
                                       **prop)
        return print_artifact(client, data, parsed_args.type_name)


class UpdateArtifact(command.Lister):
    """Update the properties of the artifact"""

    def get_parser(self, prog_name):
        parser = super(UpdateArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to update.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        ),
        parser.add_argument(
            '--remove-property', '-r',
            metavar='<key>',
            action='append',
            default=[],
            help='Property that will be removed.'
        ),
        parser.add_argument(
            '--property', '-p',
            metavar='<key=value>',
            action='append',
            default=[],
            help='Simple artifact property.'
        )
        parser.add_argument(
            '--list', '-l',
            metavar='<key=value>',
            action='append',
            default=[],
            help='Artifact list property.'
        )
        parser.add_argument(
            '--dict', '-d',
            metavar='<key=value>',
            action='append',
            default=[],
            help='Artifact dict property.'
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))

        prop = {}
        for datum in parsed_args.property:
            key, value = datum.split('=', 1)
            prop[key] = value

        for datum in parsed_args.list:
            key, value = datum.split('=', 1)
            value = strutils.split_by_commas(value)
            prop[key] = value

        for datum in parsed_args.dict:
            key, value = datum.split('=', 1)
            value = strutils.split_by_commas(value)
            prop[key] = {}
            for elem in value:
                k, v = elem.split(':', 1)
                prop[key][k] = v

        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.update(
            af_id, type_name=parsed_args.type_name,
            remove_props=parsed_args.remove_property, **prop)

        return print_artifact(client, data, parsed_args.type_name)


class DeleteArtifact(command.Command):
    """Delete the artifact"""

    def get_parser(self, prog_name):
        parser = super(DeleteArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to delete.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        ),
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        client.artifacts.delete(af_id,
                                type_name=parsed_args.type_name)


class ActivateArtifact(command.Lister):
    """Activate the artifact"""

    def get_parser(self, prog_name):
        parser = super(ActivateArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to activate.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        ),
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.activate(af_id,
                                         type_name=parsed_args.type_name)
        return print_artifact(client, data, parsed_args.type_name)


class DeactivateArtifact(command.Lister):
    """Deactivate the artifact"""

    def get_parser(self, prog_name):
        parser = super(DeactivateArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to deactivate.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        ),
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.deactivate(af_id,
                                           type_name=parsed_args.type_name)
        return print_artifact(client, data, parsed_args.type_name)


class ReactivateArtifact(command.Lister):
    """Reactivate the artifact"""

    def get_parser(self, prog_name):
        parser = super(ReactivateArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to reactivate.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        ),
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.reactivate(af_id,
                                           type_name=parsed_args.type_name)
        return print_artifact(client, data, parsed_args.type_name)


class PublishArtifact(command.Lister):
    """Publish the artifact"""

    def get_parser(self, prog_name):
        parser = super(PublishArtifact, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        ),
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to publish.',
        ),
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        ),
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        ),
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.publish(af_id,
                                        type_name=parsed_args.type_name)
        return print_artifact(client, data, parsed_args.type_name)


class AddTag(command.Lister):
    """Add tag to artifact"""

    def get_parser(self, prog_name):
        parser = super(AddTag, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        )
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to publish.',
        )
        parser.add_argument(
            'tag',
            metavar='<TAG>',
            help='Value of the tag to add to the artifact.',
        )
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        )
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.add_tag(
            af_id, tag_value=parsed_args.tag, type_name=parsed_args.type_name)
        return print_artifact(client, data, parsed_args.type_name)


class RemoveTag(command.Lister):
    """Remove tag from the artifact"""

    def get_parser(self, prog_name):
        parser = super(RemoveTag, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        )
        parser.add_argument(
            'name',
            metavar='<NAME>',
            help='Name or id of the artifact to publish.',
        )
        parser.add_argument(
            'tag',
            metavar='<TAG>',
            help='Value of the tag to remove from the artifact.',
        )
        parser.add_argument(
            '--artifact-version', '-V',
            metavar='<VERSION>',
            default='latest',
            help='Version of the artifact.',
        )
        parser.add_argument(
            '--id', '-i',
            action='store_true',
            help='Flag indicates to use artifact id instead of its name.',
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        af_id = glare_utils.get_artifact_id(client, parsed_args)
        data = client.artifacts.remove_tag(
            af_id, tag_value=parsed_args.tag, type_name=parsed_args.type_name)
        return print_artifact(client, data, parsed_args.type_name)


class TypeList(command.Lister):
    """List of type names"""

    def get_parser(self, prog_name):
        parser = super(TypeList, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        data = client.artifacts.get_type_list()

        columns = ('name', 'version')
        column_headers = [c.capitalize() for c in columns]
        return (column_headers,
                data)


class TypeSchema(command.Lister):
    """Schema of type name."""

    def get_parser(self, prog_name):
        parser = super(TypeSchema, self).get_parser(prog_name)
        parser.add_argument(
            'type_name',
            metavar='<TYPE_NAME>',
            action=TypeMapperAction,
            help='Name of artifact type.',
        )
        return parser

    def take_action(self, parsed_args):
        LOG.debug('take_action({0})'.format(parsed_args))
        client = self.app.client_manager.artifact
        data = client.artifacts.get_type_schema(
            type_name=parsed_args.type_name)['properties']

        columns = ('name', 'glare_type', 'mutable', 'required',
                   'sortable', 'filters', 'available_values')
        column_headers = [c.capitalize() for c in columns]

        table = []

        for name, values in six.iteritems(data):
            row = (
                name,
                values.get('glareType'),
                values.get('mutable', False),
                values.get('required_on_activate', True),
                values.get('sortable', False),
                values.get('filter_ops'),
                values.get('enum', '')
            )
            table.append(row)

        return (column_headers,
                table)



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# -*- coding: utf-8 -*-
#

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__),
                '..', '..')))

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
ROOT = os.path.abspath(os.path.join(BASE_DIR, "..", ".."))


def gen_ref(ver, title, names):
    refdir = os.path.join(BASE_DIR, "ref")
    pkg = "glareclient"
    if ver:
        pkg = "%s.%s" % (pkg, ver)
        refdir = os.path.join(refdir, ver)
    if not os.path.exists(refdir):
        os.makedirs(refdir)
    idxpath = os.path.join(refdir, "index.rst")
    with open(idxpath, "w") as idx:
        idx.write(("%(title)s\n"
                   "%(signs)s\n"
                   "\n"
                   ".. toctree::\n"
                   "   :maxdepth: 1\n"
                   "\n") % {"title": title, "signs": "=" * len(title)})
        for name in names:
            idx.write("   %s\n" % name)
            rstpath = os.path.join(refdir, "%s.rst" % name)
            with open(rstpath, "w") as rst:
                rst.write(("%(title)s\n"
                           "%(signs)s\n"
                           "\n"
                           ".. automodule:: %(pkg)s.%(name)s\n"
                           "   :members:\n"
                           "   :undoc-members:\n"
                           "   :show-inheritance:\n"
                           "   :noindex:\n")
                          % {"title": name.capitalize(),
                             "signs": "=" * len(name),
                             "pkg": pkg, "name": name})

gen_ref(None, "API", ["client", "exc"])
gen_ref("v1", "Glare Artifact Repository Version 1 Client Reference",
        ["client", "artifacts"])

# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'oslosphinx']

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'python-glareclient'
copyright = u'OpenStack Foundation'

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Grouping the document tree for man pages.
# List of tuples 'sourcefile', 'target', u'title', u'Authors name', 'manual'

man_pages = [
    ('man/glare', 'glare', u'Client for Glare Artifact Repository',
     [u'OpenStack Foundation'], 1),
]
# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
#html_theme = 'nature'

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project


# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    (
        'index',
        '%s.tex' % project,
        u'%s Documentation' % project,
        u'OpenStack Foundation',
        'manual'
    ),
]



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from six import StringIO
import testtools

from glareclient.common import progressbar

MOD = 'glareclient.common.progressbar'


class TestProgressBar(testtools.TestCase):

    @mock.patch(MOD + '.os')
    def test_totalsize_fileno(self, mock_os):
        mock_os.fstat.return_value.st_size = 43
        fake_file = mock.Mock()
        del fake_file.len
        fake_file.fileno.return_value = 42
        pb = progressbar.VerboseFileWrapper(fake_file)
        self.assertEqual(43, pb._totalsize)
        mock_os.fstat.assert_called_once_with(42)

    @mock.patch(MOD + '.sys')
    def test__display_progress_bar(self, mock_sys):
        fake_file = StringIO('test')  # 4 bytes
        fake_file.len = 4
        pb = progressbar.VerboseFileWrapper(fake_file)
        pb._display_progress_bar(2)  # 2 of 4 bytes = 50%
        pb._display_progress_bar(1)  # 3 of 4 bytes = 75%
        pb._display_progress_bar(1)  # 4 of 4 bytes = 100%
        expected = [
            mock.call('\r[===============>              ] 50%'),
            mock.call('\r[======================>       ] 75%'),
            mock.call('\r[=============================>] 100%'),
        ]
        self.assertEqual(expected, mock_sys.stdout.write.mock_calls)

    @mock.patch(MOD + '.sys')
    def test__display_progress_bar_unknown_len(self, mock_sys):
        fake_file = StringIO('')
        fake_file.len = 0
        pb = progressbar.VerboseFileWrapper(fake_file)
        for i in range(6):
            pb._display_progress_bar(1)
        expected = [
            mock.call('\r[-] 1 bytes'),
            mock.call('\r[\\] 2 bytes'),
            mock.call('\r[|] 3 bytes'),
            mock.call('\r[/] 4 bytes'),
            mock.call('\r[-] 5 bytes'),
            mock.call('\r[\\] 6 bytes'),
        ]
        self.assertEqual(expected, mock_sys.stdout.write.mock_calls)

    @mock.patch(MOD + '._ProgressBarBase.__init__')
    @mock.patch(MOD + '._ProgressBarBase._display_progress_bar')
    def test_read(self, mock_display_progress_bar, mock_init):
        mock_init.return_value = None
        pb = progressbar.VerboseFileWrapper()
        pb._wrapped = mock.Mock(len=42)
        pb._wrapped.read.return_value = 'ok'
        pb.read(2)
        mock_display_progress_bar.assert_called_once_with(2)




# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from glareclient.common import http
from glareclient.v1 import artifacts
from glareclient.v1 import versions


class Client(object):
    """Client for the Glare Artifact Repository v1 API.

    :param string endpoint: A user-supplied endpoint URL for the glare service.
    :param string token: Token for authentication.
    """

    def __init__(self, endpoint, **kwargs):
        """Initialize a new client for the Glare v1 API."""

        self.version = kwargs.get('version')
        self.http_client = http.construct_http_client(endpoint, **kwargs)
        self.artifacts = artifacts.Controller(self.http_client)
        self.versions = versions.VersionController(self.http_client)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import mock

import testtools

from glareclient.osc.v1 import artifacts as osc_art
from glareclient.tests.unit.osc.v1 import fakes
from glareclient.v1 import artifacts as api_art
from osc_lib.tests.utils import ParserException


class TestArtifacts(fakes.TestArtifacts):
    def setUp(self):
        super(TestArtifacts, self).setUp()
        self.artifact_mock = \
            self.app.client_manager.artifact.artifacts
        self.http = mock.MagicMock()
        self.COLUMNS = set(['id', 'name', 'owner',
                            'status', 'version', 'visibility'])


class TestListArtifacts(TestArtifacts):

    def setUp(self):
        super(TestListArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.ListArtifacts(self.app, None)
        self.COLUMNS = ['Id', 'Name', 'Version',
                        'Owner', 'Visibility', 'Status']

    def test_artifact_list(self):
        arglist = ['images']
        verify = [('type_name', 'images')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        self.assertEqual(self.COLUMNS, columns)

        self.check_parser(self.cmd, arglist, verify)

    def test_artifact_list_all(self):
        arglist = ['all']
        verify = [('type_name', 'all')]

        parsed_args = self.check_parser(self.cmd, arglist, verify)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        self.assertEqual(['Id', 'Name', 'Version', 'Type name',
                          'Owner', 'Visibility', 'Status'], columns)

        self.check_parser(self.cmd, arglist, verify)

    def test_artifact_list_with_multifilters(self):
        arglist = ['images',
                   '--filter', 'spam:spam',
                   '--filter', 'maps:maps']
        verify = [('type_name', 'images'),
                  ('filter', ['spam:spam', 'maps:maps'])]

        self.check_parser(self.cmd, arglist, verify)

    def test_artifact_list_with_sort(self):
        arglist = ['sample_artifact', '--sort', 'name:asc']
        verify = [('type_name', 'sample_artifact'),
                  ('sort', 'name:asc')]

        self.check_parser(self.cmd, arglist, verify)

    def test_artifact_list_with_multisort(self):
        arglist = ['images',
                   '--sort', 'name:desc',
                   '--sort', 'name:asc']
        verify = [('type_name', 'images'),
                  ('sort', 'name:asc')]
        self.check_parser(self.cmd, arglist, verify)

    def test_artifact_list_page_size(self):
        arglist = ['images', '--page-size', '1']
        verify = [('type_name', 'images'),
                  ('page_size', 1)]
        self.check_parser(self.cmd, arglist, verify)

    def test_artifact_list_limit(self):
        arglist = ['images', '--limit', '2']
        verify = [('type_name', 'images'),
                  ('limit', 2)]
        self.check_parser(self.cmd, arglist, verify)

    def test_artifact_list_multilimit(self):
        arglist = ['images', '--limit', '2', '--limit', '1']
        verify = [('type_name', 'images'),
                  ('limit', 1)]
        self.check_parser(self.cmd, arglist, verify)


class TestShowArtifacts(TestArtifacts):

    def setUp(self):
        super(TestShowArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.ShowArtifact(self.app, None)

    def test_artifact_show(self):
        arglist = ['images', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba']
        verify = [('type_name', 'images')]
        COLUMNS = set(['blob', 'environment', 'id', 'image',
                       'name', 'owner', 'package', 'status',
                       'template', 'version', 'visibility'])

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(COLUMNS, name_fields)

    def test_artifact_show_without_id(self):
        arglist = ['images']
        verify = [('type_name', 'images')]

        with testtools.ExpectedException(ParserException):
            self.check_parser(self.cmd, arglist, verify)

    def test_artifact_show_without_type_id(self):
        arglist = ['fc15c365-d4f9-4b8b-a090-d9e230f1f6ba']
        verify = [('type_name', 'images')]

        with testtools.ExpectedException(ParserException):
            self.check_parser(self.cmd, arglist, verify)

    def test_artifact_show_by_name(self):
        arglist = ['images', 'name1']
        verify = [('type_name', 'images'), ('id', False)]
        COLUMNS = set(['blob', 'environment', 'id', 'image',
                       'name', 'owner', 'package', 'status',
                       'template', 'version', 'visibility'])

        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(COLUMNS, name_fields)


class TestCreateArtifacts(TestArtifacts):

    def setUp(self):
        super(TestCreateArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.CreateArtifact(self.app, None)

    def test_create_artifact(self):
        arglist = ['images', 'art',
                   '--artifact-version', '0.2.4',
                   '--property', 'blah=10']
        verify = [('type_name', 'images'),
                  ('property', ['blah=10']),
                  ('artifact_version', '0.2.4')]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)

    def test_create_artifact_list_prop(self):
        arglist = ['images', 'art',
                   '--artifact-version', '0.2.4',
                   '--list', 'l=10,11,12']
        verify = [('type_name', 'images'),
                  ('list', ['l=10,11,12']),
                  ('artifact_version', '0.2.4')]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        with mock.patch.object(
                self.app.client_manager.artifact.artifacts,
                'create') as patched_create:
            self.cmd.take_action(parsed_args)
            patched_create.assert_called_once_with(
                'art',
                l=['10', '11', '12'],
                type_name='images',
                version='0.2.4')

    def test_create_artifact_dict_prop(self):
        arglist = ['images', 'art',
                   '--artifact-version', '0.2.4',
                   '--dict', 'd=a:10,b:11,c:12']
        verify = [('type_name', 'images'),
                  ('dict', ['d=a:10,b:11,c:12']),
                  ('artifact_version', '0.2.4')]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        with mock.patch.object(
                self.app.client_manager.artifact.artifacts,
                'create') as patched_create:
            self.cmd.take_action(parsed_args)
            patched_create.assert_called_once_with(
                'art',
                d={'a': '10', 'c': '12', 'b': '11'},
                type_name='images',
                version='0.2.4')

    def test_create_artifact_multiproperty(self):
        arglist = ['images', 'art',
                   '--artifact-version', '0.2.4',
                   '--property', 'blah=1',
                   '--property', 'blag=2']
        verify = [('type_name', 'images'),
                  ('property', ['blah=1', 'blag=2']),
                  ('artifact_version', '0.2.4')]
        self.check_parser(self.cmd, arglist, verify)

    def test_create_artifact_multiversion(self):
        arglist = ['images', 'art',
                   '--artifact-version', '0.2.4',
                   '--artifact-version', '0.2.5']
        verify = [('type_name', 'images'),
                  ('artifact_version', '0.2.5')]
        self.check_parser(self.cmd, arglist, verify)


class TestUpdateArtifacts(TestArtifacts):

    def setUp(self):
        super(TestUpdateArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.UpdateArtifact(self.app, None)

    def test_artifact_update(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--property', 'blah=1',
                   '--property', 'blag=2']
        verify = [('type_name', 'images'),
                  ('property', ['blah=1', 'blag=2'])]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)

    def test_update_artifact_list_prop(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--list', 'l=10,11,12']
        verify = [('type_name', 'images'),
                  ('list', ['l=10,11,12'])]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        with mock.patch.object(
                self.app.client_manager.artifact.artifacts,
                'update') as patched_update:
            self.cmd.take_action(parsed_args)
            patched_update.assert_called_once_with(
                'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                l=['10', '11', '12'],
                remove_props=[],
                type_name='images')

    def test_update_artifact_dict_prop(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--dict', 'd=a:10,b:11,c:12']
        verify = [('type_name', 'images'),
                  ('dict', ['d=a:10,b:11,c:12'])]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        with mock.patch.object(
                self.app.client_manager.artifact.artifacts,
                'update') as patched_update:
            self.cmd.take_action(parsed_args)
            patched_update.assert_called_once_with(
                'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                d={'a': '10', 'c': '12', 'b': '11'},
                remove_props=[],
                type_name='images')

    def test_artifact_update_bad(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--property', 'blah',
                   '--property', 'blah'
                   ]
        verify = [('type_name', 'images')]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        with testtools.ExpectedException(ValueError):
            self.cmd.take_action(parsed_args)

    def test_artifact_update_multiremove_prop(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--remove-property', 'prop1',
                   '--remove-property', 'prop2']
        verify = [('type_name', 'images'),
                  ('remove_property', ['prop1', 'prop2'])]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)
        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)


class TestDeleteArtifacts(TestArtifacts):

    def setUp(self):
        super(TestDeleteArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.DeleteArtifact(self.app, None)

    def test_artifact_delete(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        self.assertIsNone(self.cmd.take_action(parsed_args))


class TestActivateArtifacts(TestArtifacts):

    def setUp(self):
        super(TestActivateArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.ActivateArtifact(self.app, None)

    def test_artifact_activate(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba',
                   '--id']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)


class TestDeactivateArtifacts(TestArtifacts):

    def setUp(self):
        super(TestDeactivateArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.DeactivateArtifact(self.app, None)

    def test_artifact_deactivate(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)


class TestReactivateArtifacts(TestArtifacts):

    def setUp(self):
        super(TestReactivateArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.ReactivateArtifact(self.app, None)

    def test_artifact_rectivate(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)


class TestAddTag(TestArtifacts):

    def setUp(self):
        super(TestAddTag, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.AddTag(self.app, None)

    def test_artifact_add_tag(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id',
                   '123']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True),
                  ('tag', '123')]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)


class TestRemoveTag(TestArtifacts):

    def setUp(self):
        super(TestRemoveTag, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.RemoveTag(self.app, None)

    def test_artifact_add_tag(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id',
                   '123']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True),
                  ('tag', '123')]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)


class TestPublishArtifacts(TestArtifacts):

    def setUp(self):
        super(TestPublishArtifacts, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.PublishArtifact(self.app, None)

    def test_publish_delete(self):
        arglist = ['images',
                   'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba', '--id']
        verify = [('type_name', 'images'),
                  ('name', 'fc15c365-d4f9-4b8b-a090-d9e230f1f6ba'),
                  ('id', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        name_fields = set([column[0] for column in data])
        # Check that columns are correct
        self.assertEqual(self.COLUMNS, name_fields)


class TypeSchema(TestArtifacts):

    def setUp(self):
        super(TypeSchema, self).setUp()
        self.artifact_mock.call.return_value = \
            api_art.Controller(self.http, type_name='images')

        # Command to test
        self.cmd = osc_art.TypeSchema(self.app, None)

    def test_get_schema(self):
        arglist = ['images']
        verify = [('type_name', 'images')]
        parsed_args = self.check_parser(self.cmd, arglist, verify)
        columns, data = self.cmd.take_action(parsed_args)

        exp_columns = ['Name', 'Glare_type', 'Mutable', 'Required',
                       'Sortable', 'Filters', 'Available_values']
        exp_data = [
            (u'image', u'Blob', False, False, False, [], ''),
            (u'updated_at', u'DateTime', False, True, True,
             [u'eq', u'neq', u'in', u'gt', u'gte', u'lt', u'lte'], ''),
            (u'owner', u'String', False, False, True,
             [u'eq', u'neq', u'in'], ''),
            (u'provided_by', u'StringDict', False, False, False,
             [u'eq', u'neq', u'in'], ''),
            (u'id', u'String', False, True, True, [u'eq', u'neq', u'in'], ''),
            (u'environment', u'Blob', False, True, False, [], ''),
            (u'version', u'String', False, False, True,
             [u'eq', u'neq', u'in', u'gt', u'gte', u'lt', u'lte'], ''),
            (u'blob', u'Blob', True, False, False, [], ''),
            (u'template', u'Blob', False, True, False, [], ''),
            (u'metadata', u'StringDict', False, False, False,
             [u'eq', u'neq'], ''),
            (u'status', u'String', False, True, True, [u'eq', u'neq', u'in'],
             [u'drafted', u'active', u'deactivated', u'deleted']),
            (u'description', u'String', True, False, False,
             [u'eq', u'neq', u'in'], ''),
            (u'tags', u'StringList', True, False, False,
             [u'eq', u'neq', u'in'], ''),
            (u'activated_at', u'DateTime', False, False, True,
             [u'eq', u'neq', u'in', u'gt', u'gte', u'lt', u'lte'], ''),
            (u'supported_by', u'StringDict', False, False, False,
             [u'eq', u'neq', u'in'], ''),
            (u'visibility', u'String', False, True, True, [u'eq'], ''),
            (u'icon', u'Blob', False, False, False, [], ''),
            (u'name', u'String', False, False, True,
             [u'eq', u'neq', u'in'], ''),
            (u'license', u'String', False, False, False,
             [u'eq', u'neq', u'in'], ''),
            (u'package', u'Blob', False, False, False, [], ''),
            (u'created_at', u'DateTime', False, True, True,
             [u'eq', u'neq', u'in', u'gt', u'gte', u'lt', u'lte'], ''),
            (u'license_url', u'String', False, False, False,
             [u'eq', u'neq', u'in'], ''),
            (u'release', u'StringList', False, False, False,
             [u'eq', u'neq', u'in'], '')]

        data.sort(key=lambda x: x[0])
        exp_data.sort(key=lambda x: x[0])

        # Check that columns are correct
        self.assertEqual(exp_columns, columns)
        self.assertEqual(exp_data, data)
